#include <zmq.h>
#include <assert.h>
#include <libgen.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>
#include "utils.h"

#ifndef LOGGERS_DELAY
#define LOGGERS_DELAY 0
#endif

// Global:
char *__client;
char __logdir[STATIC_LEN];
char __regpower[STATIC_LEN];
int __loginterval;
unsigned char __stop=0;

void sighandler(int signo){
  if (signo == SIGINT){
    printf("Stopping...\n");
    __stop=1;
  }
}

int main (int argc, char *argv [])
{
  if(argc != 4){
    printf("Usage: %s <abslogdir> <client> <loginterval>",argv[0]);
    exit(1);
  }

  //----- Init global variables
  __client=argv[2];
  __loginterval=atoi(argv[3]);
  // __logdir:
  strcat(__logdir,argv[1]);
  strcat(__logdir,"/");
  strcat(__logdir,__client);
  // __regpower:
  strcat(__regpower,INA260_SYSFS);
  strcat(__regpower,"/");
  strcat(__regpower,__client);
  strcat(__regpower,"/");
  strcat(__regpower,INA260_POWER_REGISTER);

  //----- Sanity checks
  signal(SIGINT,sighandler);
  mkdirp(__logdir);
  if(__loginterval<MIN_INTERVAL){
    printf("Log interval is too small (min=%ds)\n",MIN_INTERVAL);
    exit(2);
  }
  if(FILE_EXISTS(__regpower)){ // TODO UPDATE!!
    printf("Logger cannot access to %s\n",__regpower);
    exit(3);
  }

  //----- Start logging
  printf("Logger started [client=%s,interval=%ds]\n",__client,__loginterval);

  FILE *regptr,*logptr;
  char logfilepath[STATIC_LEN]="";
  regptr=fopen("/home/loic/out.txt","r"); // TODO UPDATE!!!
  char buffer[STATIC_LEN];
  int power;
  time_t interval;
  struct timespec power_ts;
  
  while(!__stop){
    interval=INTERVAL(__loginterval);
    *logfilepath='\0'; // Clear previous path
    sprintf(logfilepath,"%s/%ld",__logdir,interval);
    logptr=fopen(logfilepath,"w");
    // Write file header:
    fprintf(logptr,"timestamp,ns,power\n");
    // Log current interval
    while((TIMESTAMP()-interval)<__loginterval){
      // Read power:
      fgets(buffer,STATIC_LEN,regptr);
      power=atoi(buffer);
      // Get power measurement timestamp:
      clock_gettime(CLOCK_REALTIME,&power_ts);
      // Write measurement in file:
      fprintf(logptr,"%d,%ld,%d\n",power_ts.tv_sec,power_ts.tv_nsec,power);
      // Reset power register file:
      fseek(regptr,0,SEEK_SET);
#if LOGGERS_DELAY > 0
      usleep(LOGGERS_DELAY*1000);
#endif
      printf("Tick\n"); fflush(stdout);
    }
    fclose(logptr);
  }

  fclose(regptr);
  return 0;
}
